@@ -24,6 +24,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #
 ##############################################################################
+import errno
 import os
 import json
 import zc.buildout
@@ -37,6 +38,9 @@ DEFAULT_IMPORT_DELIMITER = '/'
 
 @contextmanager
 def umask(mask):
+    if mask is None:
+        yield
+        return
     original = os.umask(mask)
     try:
         yield original
@@ -124,17 +128,24 @@ LOADER_TYPE_DICT = {
 }
 
 class Recipe(object):
-    mode = None
+    mode = 0777 # BBB: 0666 may have been a better default value
     loader = None
+    umask = None
 
     def __init__(self, buildout, name, options):
-        self.template = zc.buildout.download.Download(
+        template = options['template']
+        if template.startswith('inline:'):
+            template = template[7:].lstrip('\r\n')
+            self.get_template = lambda: template
+        else:
+            template = zc.buildout.download.Download(
                 buildout['buildout'],
                 hash_name=True,
             )(
-                options['template'],
+                template,
                 md5sum=options.get('md5sum'),
             )[0]
+            self.get_template = lambda: open(template).read()
         import_delimiter = options.get('import-delimiter',
             DEFAULT_IMPORT_DELIMITER)
         import_dict = {}
@@ -175,33 +186,30 @@ class Recipe(object):
         umask_value = options.get('umask')
         if umask_value:
             self.umask = int(umask_value, 8)
-        else:
-            self.umask = os.umask(0)
-            os.umask(self.umask)
 
     def install(self):
-        if os.path.lexists(self.rendered):
-            # Unlink any existing file, so umask is always applied.
+        # Unlink any existing file, so umask is always applied.
+        try:
             os.unlink(self.rendered)
+        except OSError, e:
+            if e.errno != errno.ENOENT:
+                raise
         with umask(self.umask):
             outdir = os.path.dirname(self.rendered)
             if outdir and not os.path.exists(outdir):
                 os.makedirs(outdir)
             # XXX: open doesn't allow providing a filesystem mode, so use
             # os.open and os.fdopen instead.
-            out_fd = os.open(self.rendered,
-                os.O_CREAT | os.O_TRUNC | os.O_WRONLY,
-            )
-            if self.mode is not None:
-                os.fchmod(out_fd, self.mode)
-            with os.fdopen(out_fd, 'w') as out:
+            with os.fdopen(os.open(self.rendered,
+                  os.O_CREAT | os.O_EXCL | os.O_WRONLY,
+                  self.mode), 'w') as out:
                 out.write(
                     Environment(
                         extensions=self.extension_list,
                         undefined=StrictUndefined,
                         loader=self.loader,
                     ).from_string(
-                        open(self.template).read(),
+                        self.get_template(),
                     ).render(
                         **self.context
                     )
